package supperSlonic.emrWorkflow.Builders;

import java.util.ArrayList;
import java.util.List;

import supperSlonic.emrWorkflow.JobFlowContext;
import supperSlonic.emrWorkflow.Builders.Utils.HBaseConfigurationBuilder;
import supperSlonic.emrWorkflow.Builders.Utils.HBaseDaemonsConfigurationBuilder;
import supperSlonic.emrWorkflow.Builders.Utils.HBaseInstallBootstrapAction;
import supperSlonic.emrWorkflow.Builders.Utils.HadoopConfigurationBuilder;

import com.amazonaws.services.elasticmapreduce.model.BootstrapActionConfig;
import com.amazonaws.services.elasticmapreduce.model.ScriptBootstrapActionConfig;

public class BootstrapActionBuilder extends BaseListBuilder<BootstrapActionConfig>
{
	protected BootstrapActionBuilder(JobFlowContext context)
	{
		super(context); 
	}

	@Override
	public List<BootstrapActionConfig> build()
	{
		List<BootstrapActionConfig> result = new ArrayList<BootstrapActionConfig>();
		
		//Configure Hadoop
		result.add(new HadoopConfigurationBuilder(this.context).build());
		
		Boolean ifStartHBase = this.getValueOrDefault("HBase.Start", "false").equals("true");
		if (ifStartHBase)
		{
			//Install HBase
			result.add(new HBaseInstallBootstrapAction(this.context).build());
			
			//Configure HBase Daemons
			result.add(new HBaseDaemonsConfigurationBuilder(this.context).build());
			
			//Configure HBase
			result.add(new HBaseConfigurationBuilder(this.context).build());
		}
		
		this.buildList(result);
		return result;
	}
	
	@Override
	protected BootstrapActionConfig build(int index)
	{
		String bootstrapActionPrefix = String.format("BootstrapActions.M%d.", index);		
		
		//Name
		String name = this.getValueOrNull(bootstrapActionPrefix + "Name");
		if (name == null)
		{
			return null;
		}
		
		//Path
		String path = this.getValueOrError(bootstrapActionPrefix + "Path", "Missing property \"" + bootstrapActionPrefix + "Path\". This property describes the path to the script. Example: s3://mapreduceBucket/bootstrap/action.sh"); 
		
		//Args
		ArgsBuilder argsBuilder = new ArgsBuilder(this.context, bootstrapActionPrefix);
		List<String> args = argsBuilder.build();

		ScriptBootstrapActionConfig scriptBootstrapAction = new ScriptBootstrapActionConfig(path, args);
		BootstrapActionConfig result = new BootstrapActionConfig(name, scriptBootstrapAction);
				
		System.out.println(result.toString());
		return result;
	}
}
