package supperSlonic.emrWorkflow.Builders;

import java.util.ArrayList;
import java.util.List;

import supperSlonic.emrWorkflow.JobFlowContext;
import supperSlonic.emrWorkflow.Builders.Utils.DebugStartStep;
import supperSlonic.emrWorkflow.Builders.Utils.HBaseBackupStep;
import supperSlonic.emrWorkflow.Builders.Utils.HBaseRestoreStep;
import supperSlonic.emrWorkflow.Builders.Utils.HBaseStartStep;

import com.amazonaws.services.elasticmapreduce.model.HadoopJarStepConfig;
import com.amazonaws.services.elasticmapreduce.model.StepConfig;

public class StepBuilder extends BaseListBuilder<StepConfig>
{
	protected StepBuilder(JobFlowContext context)
	{
		super(context);
	}

	@Override
	public List<StepConfig> build()
	{
		List<StepConfig> result = new ArrayList<StepConfig>();
		
		Boolean isDebug = this.getValueOrDefault("Debug.Start", "false").equals("true");
		if (isDebug)
		{
			result.add(new DebugStartStep(this.context).build());
		}
		
		Boolean ifStartHBase = this.getValueOrDefault("HBase.Start", "false").equals("true");
		if (ifStartHBase)
		{
			result.add(new HBaseStartStep(this.context).build());
		}
		
		this.buildList(result);
		
		return result;
	}

	@Override
	protected StepConfig build(int index)
	{
		return this.buildSpecificStep(index);
	}
	
	private StepConfig buildSpecificStep(int index)
	{
		String stepPrefix = String.format("Steps.M%d.", index);		
		
		String name = this.getValueOrNull(stepPrefix + "Name");
		if (name != null) {
			return this.buildHadoopStep(name, stepPrefix);
		}
		
		String restorePath = this.getValueOrNull(stepPrefix + "RestoreHBasePath");
		if (restorePath != null)
		{
			return new HBaseRestoreStep(this.context, restorePath).build();
		}
		
		String backupPath = this.getValueOrNull(stepPrefix + "BackupHBasePath");
		if (backupPath != null)
		{
			return new HBaseBackupStep(this.context, backupPath).build();
		}
		
		return null;
	}
	
	private StepConfig buildHadoopStep(String name, String stepPrefix)
	{
		//Jar
		String jar = this.getValueOrError(stepPrefix + "Jar", "Missing property \"" + stepPrefix + "Jar\". This property describes the path to the jar. Example: s3://mapreduceBucket/jar/myApplication.jar");
		HadoopJarStepConfig hadoopJarStep = new HadoopJarStepConfig(jar);
		
		//ActionOnFailure
		String actionOnFailure = this.getValueOrNull(stepPrefix + "ActionOnFailure");
		
		//MainClass
		String mainClass = this.getValueOrNull(stepPrefix + "MainClass");
		if (mainClass != null)
		{
			hadoopJarStep.setMainClass(mainClass);
		}
		
		//Args
		ArgsBuilder argsBuilder = new ArgsBuilder(this.context, stepPrefix);
		List<String> args = argsBuilder.build();
		hadoopJarStep.setArgs(args);
		
		StepConfig result = new StepConfig(name, hadoopJarStep);
		if (actionOnFailure != null) 
		{
			result.setActionOnFailure(actionOnFailure);
		}
		
		System.out.println(result.toString());
		return result;
	}
}
