package supperSlonic.emrWorkflow;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import supperSlonic.amazonConnection.entities.ResultBase;

/**
 * An instance of an object represents a context used by a JobFlow Builder.
 * It contains a reference to a properties file (Job Flow template)
 * a name of a job flow instance 
*/
public class JobFlowContext extends JobFlowContextBase
{
	public static final String EMR_JobInstanceName = "supperSlonic.aws.emr.jobInstanceName";
	public static final String EMR_JobFlowTemplate = "supperSlonic.aws.emr.jobFlowTemplate";
	
	private Properties template = null;
	
	public JobFlowContext() throws IOException
	{
		super();
	}

	public JobFlowContext(String jobInstanceName, Properties jobFlowTemplate) throws IOException 
	{
		this.instanceName = jobInstanceName;
		this.template = jobFlowTemplate;
		
		this.initDefaultConfiguration();
	}
	
	public Properties getTemplate()
	{
		return this.template;
	}
	
	public void setTemplate(Properties value)
	{
		this.template = value;
	}

	@Override
	public ResultBase isValidProperties(Properties properties) 
	{
		ResultBase result = super.isValidProperties(properties);
		if(!result.IsSuccess())
			return result;
		
		if(properties.getProperty(EMR_JobInstanceName) == null)
		{
			result.setError(ErrorResults.MissingJobInstanceNameErrorCode, ErrorResults.MissingJobInstanceNameErrorMessage);
			return result;
		}
		
		String templatePath = properties.getProperty(EMR_JobFlowTemplate);
		if(templatePath == null)
		{
			result.setError(ErrorResults.MissingTemplateFilePathErrorCode, ErrorResults.MissingTemplateFilePathErrorMessage);
			return result;
		}
		
		if(!this.checkFileExists(templatePath))
		{
			result.setError(ErrorResults.InvalidTemplateFilePathErrorCode, ErrorResults.InvalidTemplateFilePathErrorMessage);
			return result;
		}
		
		return result;
	}
	
	@Override
	protected void readProperties(Properties properties)
	{
		super.readProperties(properties);
		
		this.instanceName = properties.getProperty(EMR_JobInstanceName);
		this.template = getJobFlowTemplate(properties.getProperty(EMR_JobFlowTemplate));		
	}
	
	public static Properties getJobFlowTemplate(String filePath)
	{
		InputStream inStream = null;
		try
		{
			inStream = new FileInputStream(filePath);
			Properties result = new Properties();
			result.load(inStream);
			return result;
		}
		catch (IOException e)
		{
			System.out.println(e.getMessage());
			return null;
		}
		finally
		{
			if(inStream != null)
			{
				try
				{
					inStream.close();
				}
				catch(IOException exClose)
				{
					System.out.println(exClose.getMessage());
				}
			}
		}
	}
}