package supperSlonic.emrWorkflow;

import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;

import supperSlonic.amazonConnection.entities.Result;
import supperSlonic.amazonConnection.entities.ResultBase;
import supperSlonic.emrWorkflow.BuildersXml.Utils.ValidationErrorHandler;

public class JobFlowXmlContext extends JobFlowContextBase
{
	private Document xmlTemplate = null;
	
	public JobFlowXmlContext() throws IOException
	{
		super();
	}

	public JobFlowXmlContext(String jobInstanceName, Document jobFlowXmlTemplate) throws IOException 
	{
		this.instanceName = jobInstanceName;
		this.xmlTemplate = jobFlowXmlTemplate;
		
		this.initDefaultConfiguration();
	}
	
	public Document getTemplate()
	{
		return this.xmlTemplate;
	}
	
	public void setTemplate(Document value)
	{
		this.xmlTemplate = value;
	}
	
	public ResultBase isValidXml(String jobFlowXmlTemplatePath) 
	{
		ResultBase result = new ResultBase();
		
		if(!this.checkFileExists(jobFlowXmlTemplatePath))
		{
			result.setError(ErrorResults.InvalidTemplateFilePathErrorCode, ErrorResults.InvalidTemplateFilePathErrorMessage);
			return result;
		}
		
		Result<Document> xmlDocResult= getJobFlowXmlTemplate(jobFlowXmlTemplatePath);
		if(!xmlDocResult.IsSuccess())
		{
			result.copyError(xmlDocResult);
		}
		
		return result;
	}
	
	public static Result<Document> getJobFlowXmlTemplate(String filePath)
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(true);
		factory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaLanguage", "http://www.w3.org/2001/XMLSchema");
		factory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaSource", getSchema());

		Result<Document> result = new Result<Document>();
		try
		{
			ValidationErrorHandler errorHandler = new ValidationErrorHandler();
			
		    DocumentBuilder parser = factory.newDocumentBuilder();
		    parser.setErrorHandler(errorHandler);
		    Document xmlDoc = parser.parse(filePath);
		    
		    if(errorHandler.getErrors() == null)
		    	result.setData(xmlDoc);
		    else
		    	result.setError(ErrorResults.InvalidXmlErrorCode, errorHandler.getErrors());
		}
		catch (Exception e)
		{
			result.setError(ErrorResults.InvalidXmlErrorCode, e.getMessage());
		}
		
		return result;
	}
	
	public static String getSchema()
	{
		return Thread.currentThread().getContextClassLoader().getResource("jobFlowTemplate.xsd").toString();
	}
}
